@@ -7,14 +7,14 @@
 import sys
 
 
-def main(argv=()):
+def main(argv=()) -> int:
     # pylint: disable=import-outside-toplevel
     from xpra.platform import program_context
     with program_context("Screenshot", "Screenshot"):
         from xpra.platform.win32 import gdi_screen_capture
-        return gdi_screen_capture.main()
+        return gdi_screen_capture.main(argv)
 
 
 if __name__ == "__main__":
-    v = main()
+    v = main(sys.argv)
     sys.exit(v)
