@@ -37,7 +37,6 @@ if [ "$?" != "0" ]; then
 	echo "pandoc not found, not building HTML documentation"
 	BUILD_ARGS="${BUILD_ARGS} --without-docs"
 fi
-BUILD_ARGS="${BUILD_ARGS} --without-enc_ffmpeg"
 CONTENTS_DIR="${APP_DIR}/Contents"
 MACOS_DIR="${CONTENTS_DIR}/MacOS"
 RSCDIR="${CONTENTS_DIR}/Resources"
@@ -132,14 +131,11 @@ if [ "$?" != "0" ]; then
 fi
 echo "py2app forgets AVFoundation, do it by hand:"
 rsync -rplogt ${JHBUILD_PREFIX}/lib/python3.${PYTHON_MINOR_VERSION}/site-packages/AVFoundation ./dist/xpra.app/Contents/Resources/lib/python3.${PYTHON_MINOR_VERSION}/
-echo "fixup pkg_resources.py2_warn, gi, cffi: force include the whole packages"
-for m in pkg_resources gi cffi; do
+echo "fixup pkg_resources.py2_warn and gi: force include the whole packages"
+for m in pkg_resources gi; do
 	mpath=`python3 -c "import os;import $m;print(os.path.dirname($m.__file__))"`
 	cp -r $mpath ./dist/xpra.app/Contents/Resources/lib/python3.${PYTHON_MINOR_VERSION}/
 done
-mpath=`python3 -c "import _cffi_backend;print(_cffi_backend.__file__)"`
-cp $mpath ./dist/xpra.app/Contents/Resources/lib/python3.${PYTHON_MINOR_VERSION}/lib-dynload/
-
 echo "OK"
 popd
 echo "py2app forgets uvloop._noop"
@@ -225,13 +221,8 @@ for x in `ls "$HELPERS_DIR" | egrep -v "Python|gst-plugin-scanner"`; do
 done
 #fix for:
 # dyld: Library not loaded: @executable_path/../Resources/lib/libgstreamer-1.0.0.dylib
-pushd $RSCDIR
-ln -sf . Resources
-popd
-#fix for:
-# /Applications/Xpra.app/Contents/Resources/bin/../Resources/lib/gdk-pixbuf-2.0/2.10.0/loaders/libpixbufloader-svg.so
-pushd $RSCDIR
-ln -sf ../Resources ./Resources
+pushd $RSCDIR/libexec
+ln -sf ../../Resources ./Resources
 popd
 
 # launcher needs to be in main ("MacOS" dir) since it is launched from the custom Info.plist:
@@ -367,6 +358,8 @@ if [ "$STRIP_GSTREAMER_PLUGINS" == "1" ]; then
 	KEEP="./gstreamer-1.0.keep"
 	mkdir ${KEEP}
 	PLUGINS="app audio coreelements cutter removesilence faac faad flac oss osxaudio speex volume vorbis wav lame opus ogg gdp isomp4 matroska videoconvert x264 vpx"
+	#video sink for testing:
+	PLUGINS="${PLUGINS} autodetect osxvideo"
 	#video support:
 	PLUGINS="${PLUGINS} vpx x264 aom openh264 videoconvert videorate videoscale libav"
 	for x in $PLUGINS; do
@@ -445,6 +438,17 @@ for x in `ls *dylib | grep -v libgst | sed 's+[0-9\.]*\.dylib++g' | sed 's+-$++g
 		done
 	fi
 done
+#gstreamer dylibs are easier
+#all of them look like this: libgstXYZ-1.0.0.dylib / libgstXYZ-1.0.dylib
+for x in `ls libgst*-1.0.0.dylib`; do
+  SHORT="`echo $x | sed 's/1.0.0/1.0/g'`"
+  cmp -s "$x" "$SHORT"
+  if [ "$?" == "0" ]; then
+		echo "(re)symlinking $SHORT to $x"
+		rm $SHORT
+		ln -sf $x $SHORT
+	fi
+done
 popd
 
 
