@@ -7,7 +7,7 @@ if [ -z "${REPO_ARCH_PATH}" ]; then
 fi
 
 #find the latest version we can build:
-XPRA_TAR_XZ=`ls ../pkgs/xpra-4.4.*tar.xz | grep -v html5 | sort -V | tail -n 1`
+XPRA_TAR_XZ=`ls ../pkgs/xpra-6.*tar.xz | grep -v html5 | sort -V | tail -n 1`
 if [ -z "${XPRA_TAR_XZ}" ]; then
 	echo "no xpra source found"
 	exit 0
@@ -33,6 +33,11 @@ mk-build-deps --install --tool='apt-get -o Debug::pkgProblemResolver=yes --no-in
 #mk-build-deps --install --tool='apt-get -o Debug::pkgProblemResolver=yes --yes' debian/control
 rm -f xpra-build-deps*
 
+#install latest cython since the one Debian / Ubuntu tends to be out of date:
+DEBIAN_FRONTEND=noninteractive apt-get -y install python3-pip
+DEBIAN_FRONTEND=noninteractive apt-get -y remove cython3
+PIP_BREAK_SYSTEM_PACKAGES=1 pip3 install cython
+
 #add revision to version number to changelog
 REVISION=`PYTHONPATH=. python3 -c 'from xpra.src_info import REVISION;print(REVISION)'`
 if [ "${REVISION}" == "" ]; then
@@ -51,12 +56,13 @@ MATCH=`find $REPO_ARCH_PATH/ -name "${DEB_FILENAME}" | wc -l`
 if [ "$MATCH" != "0" ]; then
 	echo "package already exists"
 else
+  BUILD_TYPE="DEB"
 	if [ `arch` == "aarch64" ]; then
 		debuild --no-lintian -us -uc -b
 	else
 		debuild -us -uc -b
 	fi
 	ls -la ../xpra*deb
-	cp ../python*xpra*deb ../xpra*deb ../python*xpra*changes ../xpra*changes "$REPO_ARCH_PATH"
+	cp ../xpra*deb ../xpra*changes "$REPO_ARCH_PATH"
 fi
 popd
