@@ -6,24 +6,25 @@
 
 import unittest
 
-from xpra.codecs.image_wrapper import ImageWrapper
+from xpra.common import noerr
+from xpra.codecs.image import ImageWrapper
 from xpra.codecs.pillow.encoder import (
     get_encodings, encode,
     get_version, get_type, get_info,
     selftest,
-    )
+)
 
 
 class TestPillow(unittest.TestCase):
 
-
     def test_selftest(self):
         for full in (False, True):
             selftest(full)
 
     def test_module_functions(self):
-        assert get_version()>"3"
-        assert get_type()=="pillow"
+        numver = tuple(noerr(int, x) or 0 for x in get_version().split("."))
+        assert numver > (5, 0)
+        assert get_type() == "pillow"
         assert get_info()
 
     def test_resize(self):
@@ -39,7 +40,7 @@ class TestPillow(unittest.TestCase):
                 "speed" : speed,
                 "scaled-width" : 64,
                 "scaled-height" : 64,
-                })
+            })
             assert comp
 
     def test_grayscale(self):
@@ -59,7 +60,7 @@ class TestPillow(unittest.TestCase):
             "BGRA"      : 32,
             "BGR565"    : 16,
             "RLE8"      : 8,
-            }.items():
+        }.items():
             for encoding in get_encodings():
                 if encoding=="jpeg" and pixel_format!="BGRA":
                     continue
@@ -80,12 +81,18 @@ class TestPillow(unittest.TestCase):
                             image = ImageWrapper(0, 0, width, height, pixel_data, pixel_format, 32,
                                                  width*Bpp, Bpp, planes=ImageWrapper.PACKED,
                                                  thread_safe=True, palette=palette)
-                            comp = encode(encoding, image, {
+                            options = {
                                 "quality" : quality,
                                 "speed" : speed,
                                 "alpha" : transparency,
-                                })
-                            assert comp
+                            }
+                            try:
+                                comp = encode(encoding, image, options)
+                                if not comp:
+                                    raise ValueError(f"{encode} returned {comp}")
+                            except Exception as e:
+                                raise RuntimeError(f"failed to encode {image} as {encoding} with options {options}") \
+                                    from e
 
     def test_invalid_pixel_format(self):
         width = 32
@@ -126,5 +133,6 @@ class TestPillow(unittest.TestCase):
 def main():
     unittest.main()
 
+
 if __name__ == '__main__':
     main()
