@@ -6,7 +6,7 @@
 
 import unittest
 
-from xpra.util import AdHocStruct
+from xpra.util.objects import AdHocStruct
 from xpra.os_util import POSIX, OSX
 from unit.server.mixins.servermixintest_util import ServerMixinTest
 from unit.process_test_util import DisplayContext
@@ -17,10 +17,10 @@ class ClipboardMixinTest(ServerMixinTest):
     def test_clipboard(self):
         with DisplayContext():
             if POSIX and not OSX:
-                from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
+                from xpra.x11.gtk3.display_source import init_gdk_display_source
                 init_gdk_display_source()
-            from xpra.server.mixins.clipboard_server import ClipboardServer
-            from xpra.server.source.clipboard_connection import ClipboardConnection
+            from xpra.server.mixins.clipboard import ClipboardServer
+            from xpra.server.source.clipboard import ClipboardConnection
             opts = AdHocStruct()
             opts.clipboard = "yes"
             opts.clipboard_direction = "both"
