@@ -6,7 +6,7 @@
 
 import unittest
 
-from xpra.util import AdHocStruct
+from xpra.util.objects import AdHocStruct
 from unit.server.mixins.servermixintest_util import ServerMixinTest
 from unit.process_test_util import DisplayContext
 
@@ -15,8 +15,8 @@ class InputMixinTest(ServerMixinTest):
 
     def test_input(self):
         with DisplayContext():
-            from xpra.server.mixins.input_server import InputServer
-            from xpra.server.source.input_mixin import InputMixin
+            from xpra.server.mixins.input import InputServer
+            from xpra.server.source.input import InputMixin
             opts = AdHocStruct()
             self._test_mixin_class(InputServer, opts, {}, InputMixin)
 
