@@ -1,24 +1,23 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2018-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2018-2024 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import tempfile
 
-from xpra.util import envbool
-from xpra.os_util import pollwait, which, WIN32, OSX, POSIX
-from xpra.exit_codes import EXIT_STR
-from xpra.net.net_util import get_free_tcp_port
+from xpra.util.env import envbool
+from xpra.os_util import WIN32, OSX, POSIX
+from xpra.util.io import pollwait, which
+from xpra.exit_codes import exit_str
+from unit.test_util import get_free_tcp_port
 from unit.server_test_util import ServerTestUtil, log
 
-
 TEST_RFB = envbool("XPRA_TEST_RFB", not WIN32 and not OSX)
 #TEST_RFB = envbool("XPRA_TEST_RFB", False)
 VFB_INITIAL_RESOLUTION = os.environ.get("XPRA_TEST_VFB_INITIAL_RESOLUTION", "1920x1080")
 
-
 OPTIONS = [
     "windows",
     "notifications",
@@ -31,11 +30,10 @@ OPTIONS = [
     "file-transfer",
     "readonly",
     "remote-logging",
-    ]
+]
 if not WIN32:
     OPTIONS += [
-    "mmap",
-    "dbus-proxy",
+        "mmap",
     ]
 
 
@@ -49,13 +47,13 @@ class ServerMixinsOptionTestUtil(ServerTestUtil):
             tmpdir = tempfile.gettempdir()
             cls.default_xpra_args += [
                 "--socket-dirs=%s" % tmpdir,
-                ]
+            ]
             if not OSX:
                 cls.default_xpra_args += [
                     "--systemd-run=no",
                     "--pulseaudio=no",
                     "--start=xterm",
-                    ]
+                ]
         cls.display = None
         cls.xvfb = None
         cls.client_display = None
@@ -84,10 +82,15 @@ class ServerMixinsOptionTestUtil(ServerTestUtil):
                     xrandr = ["xrandr", "-s", VFB_INITIAL_RESOLUTION, "--display", self.client_display]
                     self.run_command(xrandr)
 
-
-    def _test(self, subcommand, options):
+    def _test(self, subcommand: str, options: dict):
         log("starting test server with options=%s", options)
-        args = ["--%s=%s" % (k,v) for k,v in options.items()]
+        # speed things up a bit by not loading: menu and codecs
+        if "start-new-commands" not in options:
+            options["start-new-commands"] = "no"
+        for option in ("video-decoders", "video-encoders", "csc-modules"):
+            if option not in options:
+                options[option] = "none"
+        args = ["--%s=%s" % (k, v) for k, v in options.items()]
         tcp_port = None
         xvfb = None
         if WIN32 or OSX:
@@ -100,7 +103,7 @@ class ServerMixinsOptionTestUtil(ServerTestUtil):
         else:
             display = self.find_free_display()
             connect_args = [display]
-            if subcommand=="shadow":
+            if subcommand == "shadow":
                 xvfb = self.start_Xvfb(display)
         if TEST_RFB or WIN32:
             tcp_port = get_free_tcp_port()
@@ -114,21 +117,21 @@ class ServerMixinsOptionTestUtil(ServerTestUtil):
         try:
             log("args=%s", " ".join("'%s'" % x for x in args))
             server = self.check_server(subcommand, display, *args)
-            #we should always be able to get the version:
-            client = self.run_xpra(["version"]+connect_args)
-            assert pollwait(client, 5)==0, "version client failed to connect to server with args=%s" % args
-            #run info query:
-            cmd = ["info"]+connect_args
+            # we should always be able to get the version:
+            client = self.run_xpra(["version"] + connect_args)
+            assert pollwait(client, 5) == 0, "version client failed to connect to server with args=%s" % args
+            # run info query:
+            cmd = ["info"] + connect_args
             client = self.run_xpra(cmd)
             r = pollwait(client, 20)
-            assert r==0, "info client failed and returned %s: '%s' for server with args=%s" % \
-                (r, EXIT_STR.get(r, r), args)
+            assert r == 0, "info client failed and returned %s: '%s' for server with args=%s" % \
+                           (r, exit_str(r), args)
 
             client_kwargs = {}
             if not (WIN32 or OSX):
                 env = self.get_run_env()
                 env["DISPLAY"] = self.client_display
-                client_kwargs = {"env" : env}
+                client_kwargs = {"env": env}
 
             if subcommand in ("shadow", "start-desktop") and TEST_RFB and options.get("windows", True):
                 vncviewer = which("vncviewer")
@@ -139,20 +142,22 @@ class ServerMixinsOptionTestUtil(ServerTestUtil):
                     r = pollwait(rfb_client, 10)
                     if r is not None:
                         self.show_proc_error(rfb_client,
-                                             "rfb client terminated early and returned %i for server with args=%s" % (r, args))
+                                             "rfb client terminated early and returned %i for server with args=%s" % (
+                                                 r, args))
 
             #connect a gui client:
             if WIN32 or (self.client_display and self.client_xvfb):
                 xpra_args = [
                     "attach",
-                    "--clipboard=no",       #could create loops
-                    "--notifications=no",   #may get sent to the desktop session running the tests!
-                    ]+connect_args
+                    "--clipboard=no",  # could create loops
+                    "--notifications=no",  # may get sent to the desktop session running the tests!
+                ] + connect_args
                 gui_client = self.run_xpra(xpra_args, **client_kwargs)
                 r = pollwait(gui_client, 10)
                 if r is not None:
                     self.show_proc_error(gui_client,
-                                         "gui client terminated early and returned %i : '%s' for server with args=%s" % (r, EXIT_STR.get(r, r), args))
+                                         "gui client terminated early and returned %i : '%s' for server with args=%s" % (
+                                             r, exit_str(r), args))
 
             if self.display:
                 self.stop_server(server, "exit", *connect_args)
@@ -183,12 +188,12 @@ class ServerMixinsOptionTestUtil(ServerTestUtil):
         #to test all:
         #TEST_VALUES = range(0, 2**len(OPTIONS)-1)
         #to test nothing disabled and everything disabled only:
-        TEST_VALUES = (0, 2**len(OPTIONS)-1)
+        TEST_VALUES = (0, 2 ** len(OPTIONS) - 1)
         #test every option disabled individually:
         #TEST_VALUES = tuple(2**i for i in range(len(OPTIONS)))
         for i in TEST_VALUES:
             options = {}
             for o, option in enumerate(OPTIONS):
-                options[option] = not bool((2**o) & i)
+                options[option] = not bool((2 ** o) & i)
             log("test options for %i: %s", i, options)
             self._test(subcommand, options=options)
