@@ -4,7 +4,7 @@ import sys
 
 
 def dump_xshape(xid):
-    from xpra.x11.bindings.window_bindings import X11WindowBindings, SHAPE_KIND #@UnresolvedImport
+    from xpra.x11.bindings.window import X11WindowBindings, SHAPE_KIND #@UnresolvedImport
     X11Window = X11WindowBindings()
     extents = X11Window.XShapeQueryExtents(xid)
     if not extents:
@@ -26,7 +26,7 @@ def dump_xshape(xid):
     return v
 
 def main(args):
-    from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
+    from xpra.x11.gtk3.display_source import init_gdk_display_source
     init_gdk_display_source()
     for wid in args[1:]:
         print("looking for window %s" % wid)
