@@ -2,9 +2,9 @@
 
 import sys
 import gi
-gi.require_version('Gtk', '3.0')
-gi.require_version('Gdk', '3.0')
-from gi.repository import Gtk    #pylint: disable=wrong-import-position
+gi.require_version('Gtk', '3.0')  # @UndefinedVariable
+gi.require_version('Gdk', '3.0')  # @UndefinedVariable
+from gi.repository import Gtk     # pylint: disable=wrong-import-position @UnresolvedImport
 
 width = 400
 height = 200
@@ -18,9 +18,9 @@ def main():
     window.connect("delete_event", Gtk.main_quit)
     window.realize()
     window.move(x, y)
-    from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
+    from xpra.x11.gtk3.display_source import init_gdk_display_source
     init_gdk_display_source()
-    from xpra.x11.bindings.window_bindings import X11WindowBindings
+    from xpra.x11.bindings.window import X11WindowBindings  # @UnresolvedImport
     hints = {"position" : (x, y)}
     X11WindowBindings().setSizeHints(window.get_window().get_xid(), hints)
     window.show_all()
