@@ -1,19 +1,19 @@
 #!/usr/bin/env python3
 
 import gi
-gi.require_version('Gtk', '3.0')
-gi.require_version('Gdk', '3.0')
-from gi.repository import Gtk, Gdk  #pylint: disable=wrong-import-position
 
+gi.require_version('Gtk', '3.0')  # @UndefinedVariable
+gi.require_version('Gdk', '3.0')  # @UndefinedVariable
+from gi.repository import Gtk, Gdk   # pylint: disable=wrong-import-position @UnresolvedImport
 
 def main():
 	window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
 	window.set_size_request(220, 120)
 	window.connect("delete_event", Gtk.main_quit)
-	vbox = Gtk.VBox(False, 0)
+	vbox = Gtk.VBox(homogeneous=False, spacing=0)
 
 	def add_buttons(t1, cb1, t2, cb2):
-		hbox = Gtk.HBox(True, 10)
+		hbox = Gtk.HBox(homogeneous=True, spacing=10)
 		b1 = Gtk.Button(t1)
 		def vcb1(*_args):
 			cb1()
@@ -27,9 +27,9 @@ def main():
 		vbox.pack_start(hbox, expand=False, fill=False, padding=2)
 
 	def send_maximized_wm_state(mode):
-		from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
+		from xpra.x11.gtk3.display_source import init_gdk_display_source
 		init_gdk_display_source()
-		from xpra.x11.bindings.window_bindings import constants, X11WindowBindings  #@UnresolvedImport
+		from xpra.x11.bindings.window import constants, X11WindowBindings  #@UnresolvedImport
 		X11Window = X11WindowBindings()
 		root = window.get_window().get_screen().get_root_window()
 		root_xid = root.get_xid()
@@ -38,7 +38,7 @@ def main():
 		SubstructureRedirectMask = constants["SubstructureRedirectMask"]
 		event_mask = SubstructureNotifyMask | SubstructureRedirectMask
 		X11Window.sendClientMessage(root_xid, xwin, False, event_mask, "_NET_WM_STATE", mode,
-			  "_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ", 0, 0)
+									"_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ", 0, 0)
 
 	def maximize_X11(*args):
 		send_maximized_wm_state(1)	#ADD
