@@ -1,8 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 import gi
-gi.require_version('Gtk', '3.0')
-from gi.repository import Gtk, GLib	#pylint: disable=wrong-import-position
+
+gi.require_version('Gtk', '3.0')  # @UndefinedVariable
+from gi.repository import Gtk, GLib	#pylint: disable=wrong-import-position @UnresolvedImport
 
 width = 400
 height = 200
@@ -18,9 +19,9 @@ def main():
 	window.set_title("Main")
 	window.set_size_request(width, height)
 	window.connect("delete_event", Gtk.main_quit)
-	vbox = Gtk.VBox(False, 0)
+	vbox = Gtk.VBox(homogeneous=False, spacing=0)
 	def add_button(title, callback):
-		hbox = Gtk.HBox(False, 0)
+		hbox = Gtk.HBox(homogeneous=False, spacing=0)
 		vbox.pack_start(hbox, expand=False, fill=False, padding=10)
 		btn = Gtk.Button(label=title)
 		hbox.pack_start(btn, expand=False, fill=False, padding=10)
