@@ -1,13 +1,13 @@
 #!/usr/bin/env python3
 
 import gi
-gi.require_version('Gtk', '3.0')
-from gi.repository import Gtk, GLib    #pylint: disable=wrong-import-position
+gi.require_version('Gtk', '3.0')  # @UndefinedVariable
+from gi.repository import Gtk, GLib     # pylint: disable=wrong-import-position @UnresolvedImport
 
-from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
+from xpra.x11.gtk3.display_source import init_gdk_display_source
 init_gdk_display_source()
-from xpra.x11.bindings.window_bindings import X11WindowBindings  #@UnresolvedImport
-from xpra.gtk_common.error import xsync
+from xpra.x11.bindings.window import X11WindowBindings  #@UnresolvedImport
+from xpra.gtk.error import xsync
 X11Window = X11WindowBindings()
 
 def main():
