@@ -1,10 +1,10 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2023 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from typing import Dict, Any, Callable, Tuple, Optional
-from gi.repository import GLib  # @UnresolvedImport
+from typing import Any
+from collections.abc import Callable
 
 from xpra.platform.paths import get_icon_filename
 from xpra.scripts.parsing import audio_option
@@ -12,14 +12,18 @@ from xpra.net.common import PacketType
 from xpra.net.compression import Compressed
 from xpra.net.protocol.constants import CONNECTION_LOST
 from xpra.common import FULL_INFO
-from xpra.os_util import get_machine_id, get_user_uuid, bytestostr, OSX, POSIX
-from xpra.util import envint, typedict, csv, updict
+from xpra.os_util import get_machine_id, get_user_uuid, gi_import, OSX, POSIX
+from xpra.util.objects import typedict
+from xpra.util.str_fn import csv, bytestostr
+from xpra.util.env import envint
 from xpra.client.base.stub_client_mixin import StubClientMixin
 from xpra.log import Logger
 
 avsynclog = Logger("av-sync")
 log = Logger("client", "audio")
 
+GLib = gi_import("GLib")
+
 AV_SYNC_DELTA = envint("XPRA_AV_SYNC_DELTA")
 DELTA_THRESHOLD = envint("XPRA_AV_SYNC_DELTA_THRESHOLD", 40)
 DEFAULT_AV_SYNC_DELAY = envint("XPRA_DEFAULT_AV_SYNC_DELAY", 150)
@@ -35,7 +39,7 @@ def init_audio_tagging(tray_icon) -> None:
         log("no audio module, skipping pulseaudio tagging setup")
         return
     try:
-        from xpra.audio.pulseaudio.pulseaudio_util import set_icon_path
+        from xpra.audio.pulseaudio.util import set_icon_path
         tray_icon_filename = get_icon_filename(tray_icon or "xpra")
         set_icon_path(tray_icon_filename)
     except ImportError as e:
@@ -59,56 +63,58 @@ class AudioClient(StubClientMixin):
     def __init__(self):
         super().__init__()
         self.audio_source_plugin = None
-        self.speaker_allowed : bool = False
-        self.speaker_enabled : bool = False
+        self.speaker_allowed: bool = False
+        self.speaker_enabled: bool = False
         self.speaker_codecs = []
-        self.microphone_allowed : bool = False
-        self.microphone_enabled : bool = False
+        self.microphone_allowed: bool = False
+        self.microphone_enabled: bool = False
         self.microphone_codecs = []
         self.microphone_device = None
-        self.av_sync : bool = False
-        self.av_sync_delta : int = AV_SYNC_DELTA
-        self.audio_properties : typedict = typedict()
-        #audio state:
-        self.on_sink_ready : Optional[Callable] = None
+        self.av_sync: bool = False
+        self.av_sync_delta: int = AV_SYNC_DELTA
+        self.audio_properties: typedict = typedict()
+        # audio state:
+        self.on_sink_ready: Callable | None = None
         self.audio_sink = None
-        self.audio_sink_sequence : int = 0
-        self.server_audio_eos_sequence : bool = False
+        self.audio_sink_sequence: int = 0
+        self.server_audio_eos_sequence: bool = False
         self.audio_source = None
-        self.audio_source_sequence : int = 0
-        self.audio_in_bytecount : int = 0
-        self.audio_out_bytecount : int = 0
-        self.server_av_sync : bool = False
+        self.audio_source_sequence: int = 0
+        self.audio_in_bytecount: int = 0
+        self.audio_out_bytecount: int = 0
+        self.server_av_sync: bool = False
         self.server_pulseaudio_id = ""
         self.server_pulseaudio_server = ""
-        self.server_audio_decoders : Tuple[str,...] = ()
-        self.server_audio_encoders : Tuple[str,...] = ()
-        self.server_audio_receive : bool = False
-        self.server_audio_send : bool = False
-        self.queue_used_sent : int = 0
-        #duplicated from ServerInfo mixin:
+        self.server_audio_decoders: tuple[str, ...] = ()
+        self.server_audio_encoders: tuple[str, ...] = ()
+        self.server_audio_receive: bool = False
+        self.server_audio_send: bool = False
+        self.queue_used_sent: int = 0
+        # duplicated from ServerInfo mixin:
         self._remote_machine_id = ""
 
     def init(self, opts) -> None:
         self.av_sync = opts.av_sync
         self.speaker_allowed = audio_option(opts.speaker) in ("on", "off")
-        #ie: "on", "off", "on:Some Device", "off:Some Device"
+        # ie: "on", "off", "on:Some Device", "off:Some Device"
         mic = [x.strip() for x in opts.microphone.split(":", 1)]
         self.microphone_allowed = audio_option(mic[0]) in ("on", "off")
         self.microphone_device = None
-        if self.microphone_allowed and len(mic)==2:
+        if self.microphone_allowed and len(mic) == 2:
             self.microphone_device = mic[1]
         self.audio_source_plugin = opts.audio_source
-        def nooptions(*_args) -> Tuple[str,...]:
+
+        def nooptions(*_args) -> tuple[str, ...]:
             return ()
-        audio_option_fn : Callable = nooptions
+
+        audio_option_fn: Callable = nooptions
         if self.speaker_allowed or self.microphone_allowed:
             try:
                 from xpra.audio import common
                 assert common
             except ImportError:
                 self.may_notify_audio("No Audio",
-                                      "audio subsystem is not installed\n" +
+                                      "`xpra-audio` subsystem is not installed\n"
                                       " speaker and microphone forwarding are disabled")
                 self.speaker_allowed = False
                 self.microphone_allowed = False
@@ -138,15 +144,15 @@ class AudioClient(StubClientMixin):
             self.speaker_allowed = False
         if not self.microphone_codecs:
             self.microphone_allowed = False
-        self.speaker_enabled = self.speaker_allowed and audio_option(opts.speaker)=="on"
-        self.microphone_enabled = self.microphone_allowed and audio_option(mic[0])=="on"
+        self.speaker_enabled = self.speaker_allowed and audio_option(opts.speaker) == "on"
+        self.microphone_enabled = self.microphone_allowed and audio_option(mic[0]) == "on"
         log("speaker: codecs=%s, allowed=%s, enabled=%s", encoders, self.speaker_allowed, csv(self.speaker_codecs))
         log("microphone: codecs=%s, allowed=%s, enabled=%s, default device=%s",
             decoders, self.microphone_allowed, csv(self.microphone_codecs), self.microphone_device)
         log("av-sync=%s", self.av_sync)
         if POSIX and not OSX:
             try:
-                from xpra.audio.pulseaudio.pulseaudio_util import get_info as get_pa_info
+                from xpra.audio.pulseaudio.util import get_info as get_pa_info
                 pa_info = get_pa_info()
                 log("pulseaudio info=%s", pa_info)
                 self.audio_properties.update(pa_info)
@@ -155,117 +161,97 @@ class AudioClient(StubClientMixin):
                 log.warn(" %s", e)
             except Exception:
                 log.error("Error: failed to add pulseaudio info", exc_info=True)
-        #audio tagging:
+        # audio tagging:
         init_audio_tagging(opts.tray_icon)
 
-
     def cleanup(self) -> None:
         self.stop_all_audio()
 
-
     def stop_all_audio(self) -> None:
         if self.audio_source:
             self.stop_sending_audio()
         if self.audio_sink:
             self.stop_receiving_audio()
 
-
-    def get_info(self) -> Dict[str,Any]:
-        info : Dict[str,Any] = {
-            "speaker" : self.speaker_enabled,
-            "microphone" : self.microphone_enabled,
-            "properties" : dict(self.audio_properties),
-            }
+    def get_info(self) -> dict[str, Any]:
+        info: dict[str, Any] = {
+            "speaker": self.speaker_enabled,
+            "microphone": self.microphone_enabled,
+            "properties": dict(self.audio_properties),
+        }
         ss = self.audio_source
         if ss:
             info["src"] = ss.get_info()
         ss = self.audio_sink
         if ss:
             info["sink"] = ss.get_info()
-        return {"audio" : info}
+        return {"audio": info}
 
+    def get_caps(self) -> dict[str, Any]:
+        return {
+            "av-sync": self.get_avsync_capabilities(),
+            "audio": self.get_audio_capabilities(),
+        }
 
-    def get_caps(self) -> Dict[str,Any]:
-        d : Dict[str,Any] = {}
-        avcaps = self.get_avsync_capabilities()
-        acaps = self.get_audio_capabilities()
-        #legacy flat format:
-        updict(d, "av-sync", avcaps)
-        updict(d, "audio", acaps)
-        #v4.4 namespace:
-        d["av-sync"] = avcaps
-        d["audio"] = acaps
-        return d
-
-    def get_audio_capabilities(self) -> Dict[str,Any]:
+    def get_audio_capabilities(self) -> dict[str, Any]:
         if not self.audio_properties:
             return {}
-        #we don't know if the server supports new codec names,
-        #so always add legacy names in hello:
-        caps : Dict[str,Any] = {
-            "codec-full-names"  : True,
-            "decoders"   : self.speaker_codecs,
-            "encoders"   : self.microphone_codecs,
-            "send"       : self.microphone_allowed,
-            "receive"    : self.speaker_allowed,
-            }
-        #make mypy happy about the type: convert typedict to dict with string keys
-        sp : Dict[str,Any] = dict((str(k), v) for k,v in self.audio_properties.items())
-        if FULL_INFO<2:
-            #only expose these specific keys:
-            sp = dict((k,v) for k,v in sp.items() if k in (
-                "encoders", "decoders", "muxers", "demuxers", "bundle-metadata", "ogg-latency-fix",
-                ))
+        caps: dict[str, Any] = {
+            "decoders": self.speaker_codecs,
+            "encoders": self.microphone_codecs,
+            "send": self.microphone_allowed,
+            "receive": self.speaker_allowed,
+        }
+        # make mypy happy about the type: convert typedict to dict with string keys
+        sp: dict[str, Any] = {str(k): v for k, v in self.audio_properties.items()}
+        if FULL_INFO < 2:
+            # only expose these specific keys:
+            sp = {k: v for k, v in sp.items() if k in (
+                "encoders", "decoders", "muxers", "demuxers",
+            )}
         caps.update(sp)
         log("audio capabilities: %s", caps)
         return caps
 
-    def get_avsync_capabilities(self) -> Dict[str,Any]:
+    def get_avsync_capabilities(self) -> dict[str, Any]:
         if not self.av_sync:
             return {}
         delay = max(0, DEFAULT_AV_SYNC_DELAY + AV_SYNC_DELTA)
         return {
-            ""              : True,
-            "enabled"       : True,
-            "delay.default" : delay,
-            "delay"         : delay,
-            }
+            "": True,
+            "enabled": True,
+            "delay.default": delay,
+            "delay": delay,
+        }
 
-
-    def parse_server_capabilities(self, c : typedict) -> bool:
+    def parse_server_capabilities(self, c: typedict) -> bool:
         self.server_av_sync = c.boolget("av-sync.enabled")
         avsynclog("av-sync: server=%s, client=%s", self.server_av_sync, self.av_sync)
-        audio = c.get("audio")
-        if audio and isinstance(audio, dict):
-            c = typedict(audio)
-            prefix = ""
-        else:
-            prefix = "sound."
-        self.server_pulseaudio_id = c.strget(f"{prefix}pulseaudio.id")
-        self.server_pulseaudio_server = c.strget(f"{prefix}pulseaudio.server")
-        self.server_audio_decoders = c.strtupleget(f"{prefix}decoders")
-        self.server_audio_encoders = c.strtupleget(f"{prefix}encoders")
-        self.server_audio_receive = c.boolget(f"{prefix}receive")
-        self.server_audio_send = c.boolget(f"{prefix}send")
+        audio = typedict(c.dictget("audio") or {})
+        self.server_pulseaudio_id = audio.strget("pulseaudio.id")
+        self.server_pulseaudio_server = audio.strget("pulseaudio.server")
+        self.server_audio_decoders = audio.strtupleget("decoders")
+        self.server_audio_encoders = audio.strtupleget("encoders")
+        self.server_audio_receive = audio.boolget("receive")
+        self.server_audio_send = audio.boolget("send")
         log("pulseaudio id=%s, server=%s, audio decoders=%s, audio encoders=%s, receive=%s, send=%s",
-                 self.server_pulseaudio_id, self.server_pulseaudio_server,
-                 csv(self.server_audio_decoders), csv(self.server_audio_encoders),
-                 self.server_audio_receive, self.server_audio_send)
+            self.server_pulseaudio_id, self.server_pulseaudio_server,
+            csv(self.server_audio_decoders), csv(self.server_audio_encoders),
+            self.server_audio_receive, self.server_audio_send)
         if self.server_audio_send and self.speaker_enabled:
             self.show_progress(90, "starting speaker forwarding")
             self.start_receiving_audio()
         if self.server_audio_receive and self.microphone_enabled:
-            #call via idle_add because we may query X11 properties
-            #to find the pulseaudio server:
+            # call via idle_add because we may query X11 properties
+            # to find the pulseaudio server:
             GLib.idle_add(self.start_sending_audio)
         return True
 
-
     ######################################################################
     # audio:
 
-    def may_notify_audio(self, summary:str, body:str) -> None:
-        #overridden in UI client subclass
+    def may_notify_audio(self, summary: str, body: str) -> None:
+        # overridden in UI client subclass
         pass
 
     def audio_loop_check(self, mode="speaker") -> bool:
@@ -273,19 +259,19 @@ class AudioClient(StubClientMixin):
         if ALLOW_SOUND_LOOP:
             return True
         if self._remote_machine_id:
-            if self._remote_machine_id!=get_machine_id():
-                #not the same machine, so OK
+            if self._remote_machine_id != get_machine_id():
+                # not the same machine, so OK
                 return True
-            if self._remote_uuid!=get_user_uuid():
-                #different user, assume different pulseaudio server
+            if self._remote_uuid != get_user_uuid():
+                # different user, assume different pulseaudio server
                 return True
-        #check pulseaudio id if we have it
+        # check pulseaudio id if we have it
         pulseaudio_id = self.audio_properties.get("pulseaudio", {}).get("id")
         if not pulseaudio_id or not self.server_pulseaudio_id:
-            #not available, assume no pulseaudio so no loop?
+            # not available, assume no pulseaudio so no loop?
             return True
-        if self.server_pulseaudio_id!=pulseaudio_id:
-            #different pulseaudio server
+        if self.server_pulseaudio_id != pulseaudio_id:
+            # different pulseaudio server
             return True
         msgs = loop_warning_messages(mode)
         summary = msgs[0]
@@ -315,7 +301,7 @@ class AudioClient(StubClientMixin):
                 return
             ss = self.audio_source
             if ss:
-                if ss.get_state()=="active":
+                if ss.get_state() == "active":
                     log.error("Error: microphone forwarding is already active")
                     enabled = True
                     return
@@ -323,7 +309,7 @@ class AudioClient(StubClientMixin):
             else:
                 enabled = self.start_audio_source(device)
         finally:
-            if enabled!=self.microphone_enabled:
+            if enabled != self.microphone_enabled:
                 self.microphone_enabled = enabled
                 self.emit("microphone-changed")
             log("start_sending_audio(%s) done, microphone_enabled=%s", device, enabled)
@@ -331,9 +317,11 @@ class AudioClient(StubClientMixin):
     def start_audio_source(self, device=None) -> bool:
         log("start_audio_source(%s)", device)
         assert self.audio_source is None
+
         def audio_source_state_changed(*_args):
             self.emit("microphone-changed")
-        #find the matching codecs:
+
+        # find the matching codecs:
         matching_codecs = get_matching_codecs(self.microphone_codecs, self.server_audio_decoders)
         log("start_audio_source(%s) matching codecs: %s", device, csv(matching_codecs))
         if not matching_codecs:
@@ -361,19 +349,19 @@ class AudioClient(StubClientMixin):
             log.estr(e)
             return False
 
-    def new_stream(self, audio_source, codec:str):
+    def new_stream(self, audio_source, codec: str):
         log("new_stream(%s)", codec)
-        if self.audio_source!=audio_source:
+        if self.audio_source != audio_source:
             log("dropping new-stream signal (current source=%s, signal source=%s)", self.audio_source, audio_source)
             return
         codec = codec or audio_source.codec
         audio_source.codec = codec
-        #tell the server this is the start:
+        # tell the server this is the start:
         self.send("sound-data", codec, "",
                   {
-                   "start-of-stream"    : True,
-                   "codec"              : codec,
-                   })
+                      "start-of-stream": True,
+                      "codec": codec,
+                  })
 
     def stop_sending_audio(self) -> None:
         """ stop the audio source and emit client signal """
@@ -386,11 +374,11 @@ class AudioClient(StubClientMixin):
         if ss is None:
             log.warn("Warning: cannot stop audio capture which has not been started")
             return
-        #tell the server to stop:
+        # tell the server to stop:
         self.send("sound-data", ss.codec or "", "", {
-            "end-of-stream" : True,
-            "sequence"      : ss.sequence,
-            })
+            "end-of-stream": True,
+            "sequence": ss.sequence,
+        })
         self.audio_source_sequence += 1
         ss.cleanup()
 
@@ -408,27 +396,29 @@ class AudioClient(StubClientMixin):
                 return
             if not self.audio_loop_check("speaker"):
                 return
-            #choose a codec:
+            # choose a codec:
             matching_codecs = get_matching_codecs(self.speaker_codecs, self.server_audio_encoders)
             log("start_receiving_audio() matching codecs: %s", csv(matching_codecs))
             if not matching_codecs:
                 self.no_matching_codec_error("speaker", self.server_audio_encoders, self.speaker_codecs)
                 return
             codec = matching_codecs[0]
+
             def sink_ready(*args):
                 scodec = codec
                 log("sink_ready(%s) codec=%s (server codec name=%s)", args, codec, scodec)
                 self.send("sound-control", "start", scodec)
                 return False
+
             self.on_sink_ready = sink_ready
             enabled = self.start_audio_sink(codec)
         finally:
-            if self.speaker_enabled!=enabled:
+            if self.speaker_enabled != enabled:
                 self.speaker_enabled = enabled
                 self.emit("speaker-changed")
             log("start_receiving_audio() done, speaker_enabled=%s", enabled)
 
-    def stop_receiving_audio(self, tell_server:bool=True) -> None:
+    def stop_receiving_audio(self, tell_server: bool = True) -> None:
         """
             ask the server to stop sending audio
             and toggle the flag so that we ignore further packets
@@ -441,7 +431,7 @@ class AudioClient(StubClientMixin):
             self.emit("speaker-changed")
         if not ss:
             return
-        if tell_server and ss.sequence==self.audio_sink_sequence:
+        if tell_server and ss.sequence == self.audio_sink_sequence:
             self.send("sound-control", "stop", self.audio_sink_sequence)
         self.audio_sink_sequence += 1
         self.send("sound-control", "new-sequence", self.audio_sink_sequence)
@@ -450,28 +440,31 @@ class AudioClient(StubClientMixin):
         ss.cleanup()
         log("stop_receiving_audio(%s) done", tell_server)
 
-    def audio_sink_state_changed(self, audio_sink, state:str) -> None:
-        if audio_sink!=self.audio_sink:
+    def audio_sink_state_changed(self, audio_sink, state: str) -> None:
+        if audio_sink != self.audio_sink:
             log("audio_sink_state_changed(%s, %s) not the current sink, ignoring it", audio_sink, state)
             return
         log("audio_sink_state_changed(%s, %s) on_sink_ready=%s", audio_sink, state, self.on_sink_ready)
-        if bytestostr(state)=="ready" and self.on_sink_ready:
+        if bytestostr(state) == "ready" and self.on_sink_ready:
             if not self.on_sink_ready():
                 self.on_sink_ready = None
         self.emit("speaker-changed")
-    def audio_sink_bitrate_changed(self, audio_sink, bitrate:int) -> None:
-        if audio_sink!=self.audio_sink:
+
+    def audio_sink_bitrate_changed(self, audio_sink, bitrate: int) -> None:
+        if audio_sink != self.audio_sink:
             log("audio_sink_bitrate_changed(%s, %s) not the current sink, ignoring it", audio_sink, bitrate)
             return
         log("audio_sink_bitrate_changed(%s, %s)", audio_sink, bitrate)
-        #not shown in the UI, so don't bother with emitting a signal:
-        #self.emit("speaker-changed")
+        # not shown in the UI, so don't bother with emitting a signal:
+        # self.emit("speaker-changed")
+
     def audio_sink_error(self, audio_sink, error) -> None:
-        log("audio_sink_error(%s, %s) exit_code=%s, current sink=%s", audio_sink, error, self.exit_code, self.audio_sink)
+        log("audio_sink_error(%s, %s) exit_code=%s, current sink=%s",
+            audio_sink, error, self.exit_code, self.audio_sink)
         if self.exit_code is not None:
-            #exiting
+            # exiting
             return
-        if audio_sink!=self.audio_sink:
+        if audio_sink != self.audio_sink:
             log("audio_sink_error(%s, %s) not the current sink, ignoring it", audio_sink, error)
             return
         estr = bytestostr(error).replace("gst-resource-error-quark: ", "")
@@ -479,11 +472,12 @@ class AudioClient(StubClientMixin):
         log.warn("Error: stopping speaker:")
         log.warn(" %s", estr)
         self.stop_receiving_audio()
+
     def audio_process_stopped(self, audio_sink, *args) -> None:
         if self.exit_code is not None:
-            #exiting
+            # exiting
             return
-        if audio_sink!=self.audio_sink:
+        if audio_sink != self.audio_sink:
             log("audio_process_stopped(%s, %s) not the current sink, ignoring it", audio_sink, args)
             return
         log.warn("Warning: the audio process has stopped")
@@ -492,20 +486,20 @@ class AudioClient(StubClientMixin):
     def audio_sink_exit(self, audio_sink, *args) -> None:
         log("audio_sink_exit(%s, %s) audio_sink=%s", audio_sink, args, self.audio_sink)
         if self.exit_code is not None:
-            #exiting
+            # exiting
             return
         ss = self.audio_sink
-        if audio_sink!=ss:
+        if audio_sink != ss:
             log("audio_sink_exit() not the current sink, ignoring it")
             return
         if ss and ss.codec:
-            #the mandatory "I've been naughty warning":
-            #we use the "codec" field as guard to ensure we only print this warning once..
+            # the mandatory "I've been naughty warning":
+            # we use the "codec" field as guard to ensure we only print this warning once…
             log.warn("Warning: the %s audio sink has stopped", ss.codec)
             ss.codec = ""
         self.stop_receiving_audio()
 
-    def start_audio_sink(self, codec:str) -> bool:
+    def start_audio_sink(self, codec: str) -> bool:
         log("start_audio_sink(%s)", codec)
         assert self.audio_sink is None, "audio sink already exists!"
         try:
@@ -530,7 +524,7 @@ class AudioClient(StubClientMixin):
 
     def new_audio_buffer(self, audio_source, data, metadata, packet_metadata=()) -> None:
         log("new_audio_buffer(%s, %s, %s, %s)", audio_source, len(data or ()), metadata, packet_metadata)
-        if audio_source.sequence<self.audio_source_sequence:
+        if audio_source.sequence < self.audio_source_sequence:
             log("audio buffer dropped: old sequence number: %s (current is %s)",
                 audio_source.sequence, self.audio_source_sequence)
             return
@@ -539,7 +533,7 @@ class AudioClient(StubClientMixin):
             self.audio_out_bytecount += len(x)
         metadata["sequence"] = audio_source.sequence
         if packet_metadata:
-            #the packet metadata is already compressed:
+            # the packet metadata is already compressed:
             packet_metadata = Compressed("packet metadata", packet_metadata, can_inline=True)
         self.send_audio_data(audio_source, data, metadata, packet_metadata)
 
@@ -551,26 +545,26 @@ class AudioClient(StubClientMixin):
     def send_audio_sync(self, v) -> None:
         self.send("sound-control", "sync", v)
 
-
     ######################################################################
-    #packet handlers
-    def _process_sound_data(self, packet : PacketType) -> None:
+    # packet handlers
+
+    def _process_sound_data(self, packet: PacketType) -> None:
         codec, data, metadata = packet[1:4]
         codec = bytestostr(codec)
         metadata = typedict(metadata)
         if data:
             self.audio_in_bytecount += len(data)
-        #verify sequence number if present:
+        # verify sequence number if present:
         seq = metadata.intget("sequence", -1)
-        if self.audio_sink_sequence>0 and 0<=seq<self.audio_sink_sequence:
+        if self.audio_sink_sequence > 0 and 0 <= seq < self.audio_sink_sequence:
             log("ignoring audio data with old sequence number %s (now on %s)", seq, self.audio_sink_sequence)
             return
 
         if not self.speaker_enabled:
             if metadata.boolget("start-of-stream"):
-                #server is asking us to start playing audio
+                # server is asking us to start playing audio
                 if not self.speaker_allowed:
-                    #no can do!
+                    # no can do!
                     log.warn("Warning: cannot honour the request to start the speaker")
                     log.warn(" speaker forwarding is disabled")
                     self.stop_receiving_audio(True)
@@ -592,31 +586,31 @@ class AudioClient(StubClientMixin):
             log("server sent end-of-stream for sequence %s, closing audio pipeline", seq)
             self.stop_receiving_audio(False)
             return
-        if codec!=ss.codec:
+        if codec != ss.codec:
             log.error("Error: audio codec change is not supported!")
             log.error(" stream tried to switch from %s to %s", ss.codec, codec)
             self.stop_receiving_audio()
             return
-        if ss.get_state()=="stopped":
+        if ss.get_state() == "stopped":
             log("audio data received, audio sink is stopped - telling server to stop")
             self.stop_receiving_audio()
             return
-        #the server may send packet_metadata, which is pushed before the actual audio data:
+        # the server may send packet_metadata, which is pushed before the actual audio data:
         packet_metadata = ()
-        if len(packet)>4:
+        if len(packet) > 4:
             packet_metadata = packet[4]
-        #(some packets (ie: sos, eos) only contain metadata)
+        # (some packets (ie: sos, eos) only contain metadata)
         if data or packet_metadata:
             ss.add_data(data, metadata, packet_metadata)
         if self.av_sync and self.server_av_sync:
             qinfo = typedict(ss.get_info()).dictget("queue")
-            queue_used = typedict(qinfo or {}).intget("cur", None)
-            if queue_used is None:
+            queue_used = typedict(qinfo or {}).intget("cur", -1)
+            if queue_used < 0:
                 return
-            delta = (self.queue_used_sent or 0)-queue_used
-            #avsynclog("server audio sync: queue info=%s, last sent=%s, delta=%s",
+            delta = (self.queue_used_sent or 0) - queue_used
+            # avsynclog("server audio sync: queue info=%s, last sent=%s, delta=%s",
             #    dict((k,v) for (k,v) in info.items() if k.startswith("queue")), self.queue_used_sent, delta)
-            if self.queue_used_sent is None or abs(delta)>=DELTA_THRESHOLD:
+            if self.queue_used_sent is None or abs(delta) >= DELTA_THRESHOLD:
                 avsynclog("server audio sync: sending updated queue.used=%i (was %s)",
                           queue_used, (self.queue_used_sent or "unset"))
                 self.queue_used_sent = queue_used
@@ -625,8 +619,7 @@ class AudioClient(StubClientMixin):
                     avsynclog(" adjusted value=%i with sync delta=%i", v, self.av_sync_delta)
                 self.send_audio_sync(v)
 
-
     def init_authenticated_packet_handlers(self) -> None:
         log("init_authenticated_packet_handlers()")
-        #these handlers can run directly from the network thread:
+        # these handlers can run directly from the network thread:
         self.add_packet_handler("sound-data", self._process_sound_data, False)
